//
//  Point3d.hpp	3D Vector Math Class/Template
//
//  Copyright by Copac I.T. 1997. All rights reserved.
//

// with some tiny adjustments done by Jeroen Bakker
// file : Point3d.h

#ifndef point3d_h_included
#define point3d_h_included

class Point3d
{
public:
	float x,y,z;
  inline Point3d() {}
  inline Point3d( float a[3] ) : x(a[0]), y(a[1]), z(a[2]) {}
  inline Point3d( float a, float b, float c ) { x=a; y=b; z=c; }
  inline Point3d( Point3d const &v ) : x(v.x), y(v.y), z(v.z) {}

  Point3d& operator  = ( Point3d const & );
  Point3d& operator += ( Point3d const & );
  Point3d& operator -= ( Point3d const & );
  Point3d& operator /= ( float const );		// scalar
  Point3d& operator *= ( float const );		// scalar
  Point3d& operator %= ( Point3d const & );		// uit-product (cross)

  Point3d operator + () const;
  Point3d operator - () const;

  inline float	SqrLength() const { return (x*x+y*y+z*z); }
  inline float	Length() const { return (float)sqrt(x*x+y*y+z*z); }
  inline void	Normalize() { float l=Length(); x/=l; y/=l; z/=l; }

};

/////////////////////////////////////////////////////////////////////////////

#define ZeroVector() Point3d(0,0,0)

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

inline Point3d& Point3d::operator = ( Point3d const &v )
{
  x = v.x;
  y = v.y;
  z = v.z;
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d& Point3d::operator += ( Point3d const &v )
{
  x += v.x;
  y += v.y;
  z += v.z;
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d& Point3d::operator -= ( Point3d const &v )
{
  x -= v.x;
  y -= v.y;
  z -= v.z;
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d& Point3d::operator *= ( float const d )
{
  x *= d;
  y *= d;
  z *= d;
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d& Point3d::operator /= ( float const d )
{
  x /= d;
  y /= d;
  z /= d;
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d Point3d::operator + () const
{
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d Point3d::operator - () const
{
  return Point3d( -x, -y, -z );
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d& Point3d::operator %= ( Point3d const &v )	// uit-product (cross-)
{
  float _x=x,_y=y;
  x = v.y *  z - v.z * _y;
  y = v.z * _x - v.x *  z;
  z = v.x * _y - v.y * _x;
  return (*this);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
//				Friends					   //
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

inline Point3d operator * ( float const d, Point3d const &v )
{
  return Point3d(v.x*d,v.y*d,v.z*d);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d operator * ( Point3d const &v, float const d )
{
  return Point3d(v.x*d,v.y*d,v.z*d);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d operator / ( Point3d const &v, float const d )
{
  return Point3d(v.x/d,v.y/d,v.z/d);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d operator + ( Point3d const &v1, Point3d const &v2 )
{
  return Point3d(v1.x+v2.x,v1.y+v2.y,v1.z+v2.z);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d operator - ( Point3d const &v1, Point3d const &v2 )
{
  return Point3d(v1.x-v2.x,v1.y-v2.y,v1.z-v2.z);
}

/////////////////////////////////////////////////////////////////////////////

inline Point3d operator % ( Point3d const &v1, Point3d const &v2 )
{
  return Point3d(v1.y*v2.z-v1.z*v2.y,v1.z*v2.x-v1.x*v2.z,v1.x*v2.y-v1.y*v2.x);
}

/////////////////////////////////////////////////////////////////////////////

inline float operator * ( Point3d const &v1, Point3d const &v2 )
{
  return (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

#endif // point3d_h_included