// file : QuaternionRenderer.cpp

#include "QuaternionRenderer.h"
#include <math.h>

QuaternionRenderer::QuaternionRenderer()
{
	setSize( 320, 200 );
	setDescriptor( 0 );
}

void QuaternionRenderer::render(QuaternionDescriptor *theDescriptor)
{
	setDescriptor(theDescriptor);
	render();	
}

void QuaternionRenderer::render()
{
	int offset, x, y;
	preRender();
	
	offset=0;
	for (y=0;y<height;y++)
	for (x=0;x<width;x++)
		outputBuffer[offset++] = trace2d( x, y );
}

void QuaternionRenderer::preRender()
{
//  sx=width;sy=height;
	d = bound / (Point3d(width, height, zant));
//  dx=(bound.max.x-bound.min.x)/width;
//  dy=(bound.max.y-bound.min.y)/height;
//  dz=(bound.max.z-bound.min.z)/zant;
  dzf1=d.z/zant1;

  orig=bound.center();

  cosx=(float)cos(descriptorToCalculate->alpha);
  cosy=(float)cos(descriptorToCalculate->beta);
  cosz=(float)cos(descriptorToCalculate->gamma);
  sinx=(float)sin(descriptorToCalculate->alpha);
  siny=(float)sin(descriptorToCalculate->beta);
  sinz=(float)sin(descriptorToCalculate->gamma);

  rotateValues();
}

void QuaternionRenderer::setDescriptor( QuaternionDescriptor *aDescriptor )
{
	descriptorToCalculate = aDescriptor;
}


void QuaternionRenderer::setSize( int newWidth, int newHeight )
{
	width = newWidth;
	height = newHeight;
}

void QuaternionRenderer::setOutputBuffer( char* aBuffer )
{
	outputBuffer = aBuffer;
}

void QuaternionRenderer::rotate3d(Point3d& v)
{ // use matrix not abc rotations...
  v-=orig;
  float xy=v.y*cosx - v.z*sinx;
  float xz=v.y*sinx + v.z*cosx;
  float xx=v.x;
  v.x=xx;
  v.y=xy;
  v.z=xz;
  float yx=v.x*cosy + v.z*siny;
  float yz=v.x*siny + v.z*cosy;
  v.x=yx;
  v.z=yz;
  float zx=v.x*cosz - v.y*sinz;
  float zy=v.x*sinz + v.y*cosz;
  v.x=zx;
  v.y=zy;
  v+=orig;
}

void QuaternionRenderer::rotateValues()
{
  rmin=bound.min;
  rotate3d(rmin);
  temp=Point3d( bound.max.x, bound.min.y, bound.min.z);
  rotate3d(temp);
  dx = (temp-rmin) / width;
  temp=Point3d ( bound.min.x, bound.max.y, bound.min.z);
  rotate3d(temp);
  dy = (temp-rmin) / height;
  temp=Point3d( bound.min.x, bound.min.y, bound.max.z);
  rotate3d(temp);
  dz=(temp-rmin)/zant;
  dz1=dz/zant1;
}

float QuaternionRenderer::trace3d( float x, float y, float z )
{
  float lengde;
  float temp;
  float w=descriptorToCalculate->wk;
  int m=0;
  do {
	temp=x+x;
	x=x*x - y*y - z*z - w*w + descriptorToCalculate->cr;
	y=temp*y+descriptorToCalculate->ci;
	z=temp*z+descriptorToCalculate->cj;
	w=temp*w+descriptorToCalculate->ck;

	m++;
	lengde=x*x + y*y + z*z + w*w;
  } while ((m<iter) && (lengde<2));
 return lengde;
}

char QuaternionRenderer::trace2d( float px, float py )
{
  int pz, pz1;
  float l;
	
	temp = rmin + dx*px + dy*py;
	pz=0;
	do {
		temp += dz;
	  l=trace3d(temp.x,temp.y,temp.z);
	  pz++;
	} while ((l>2) && (pz<zant));
	pz1=0;
	do {
	  pz1++;
	  temp -= dz1;
	  l=trace3d(temp.x,temp.y,temp.z);
	} while (l<2);
	if (pz < zant)
	{
	  return (char)((bound.min.z+pz*d.z-pz1*dzf1)*100)+128;
	}
	else return 255;

}