// file QuaternionSettingsWindow.cpp

#include "QuaternionSettingsWindow.h"
#include <stdio.h>

#define IDAVIT_RENDER	'IREN'
#define IDAVIT_UPDATESLIDERS	'UDSL'

QuaternionSettingsWindow::QuaternionSettingsWindow() :
	BWindow( BRect( 10,10, 210,460 ), "Settings", B_BORDERED_WINDOW, 0 )
{
	
	numberOfSlidersHooked=0;
	numberOfButtonsHooked=0;

	aView = new BView(BRect( 0,0,200,450 ), 0, B_FOLLOW_ALL_SIDES, 0 );
	AddChild(aView);
	aView -> SetViewColor( 200,255,255,0 );
	

	sliderAlpha = addSlider( "Alpha:",0,359, "0", "359");
	sliderBeta = addSlider( "Beta:",0,359, "0", "359");
	sliderGamma = addSlider( "Gamma:",0,359, "0", "359");
	sliderCr = addSlider( "Cr:",-100,100, "-1", "1");
	sliderCi = addSlider( "Ci:",-100,100, "-1", "1");
	sliderCj = addSlider( "Cj:",-100,100, "-1", "1");
	sliderCk = addSlider( "Ck:",-100,100, "-1", "1");
	sliderWk = addSlider( "Wk:",-100,100, "-1", "1");
	buttonQuit = addButton("Close", new BMessage(B_QUIT_REQUESTED));
	buttonRender = addButton("Render", new BMessage(IDAVIT_RENDER));

	updateValueStrings();	
}

bool QuaternionSettingsWindow::QuitRequested()
{
	BWindow::QuitRequested();
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

BSlider* QuaternionSettingsWindow::addSlider( char* labelName, int min, int max, char* minText, char* maxText)
{
	BSlider *temporarySliderKeeper;
	
	temporarySliderKeeper = new BSlider(
		BRect(5,5+(numberOfSlidersHooked*48),195,5+((numberOfSlidersHooked+1)*48)),
		0, labelName, new BMessage(IDAVIT_UPDATESLIDERS), 
		min, max,B_BLOCK_THUMB,B_FOLLOW_LEFT_RIGHT
		);
	temporarySliderKeeper->SetLimitLabels(minText, maxText);
	aView->AddChild(temporarySliderKeeper);
	numberOfSlidersHooked++;
	return temporarySliderKeeper;
}

BButton* QuaternionSettingsWindow::addButton(char* labelName, BMessage* theMessage)
{
	BButton *temporaryButtonKeeper;
	int x = 195-numberOfButtonsHooked*85;
	int y = 445;
	temporaryButtonKeeper = new BButton(
		BRect(x-80,y-24,x,y),NULL, labelName, theMessage, B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM);
	aView -> AddChild(temporaryButtonKeeper);
	numberOfButtonsHooked++;

	return temporaryButtonKeeper;
}

void QuaternionSettingsWindow::MessageReceived(BMessage *theMessage)
{
	switch(theMessage->what)
	{
		case IDAVIT_RENDER:
			render();		
			break;
		case IDAVIT_UPDATESLIDERS:
			updateValueStrings();
			render();
			break;
		default:
			BWindow::MessageReceived(theMessage);
	}
}

void QuaternionSettingsWindow::setOutputWindow( QuaternionOutputWindow *source )
{
	theOutputWindow = source;
}

void QuaternionSettingsWindow::render()
{
	buildDescriptor();
	theOutputWindow->render( &theDescriptor );
}

void QuaternionSettingsWindow::updateValueStrings()
{
	char label[40];
	sprintf(label, "Alpha: %d", (int)sliderAlpha->Value());
	sliderAlpha->SetLabel(label);
	sprintf(label, "Beta: %d", (int)sliderBeta->Value());
	sliderBeta->SetLabel(label);
	sprintf(label, "Gamma: %d", (int)sliderGamma->Value());
	sliderGamma->SetLabel(label);

	sprintf(label, "Cr: %f", sliderCr->Value()/100.0f);
	sliderCr->SetLabel(label);
	sprintf(label, "Ci: %f", sliderCi->Value()/100.0f);
	sliderCi->SetLabel(label);
	sprintf(label, "Cj: %f", sliderCj->Value()/100.0f);
	sliderCj->SetLabel(label);
	sprintf(label, "Ck: %f", sliderCk->Value()/100.0f);
	sliderCk->SetLabel(label);
	sprintf(label, "Wk: %f", sliderWk->Value()/100.0f);
	sliderWk->SetLabel(label);
}

void QuaternionSettingsWindow::buildDescriptor()
{
	theDescriptor.alpha=sliderAlpha->Value()/180.0f*M_PI;
	theDescriptor.beta=sliderBeta->Value()/180.0f*M_PI;
	theDescriptor.gamma=sliderGamma->Value()/180.0f*M_PI;
	
	theDescriptor.cr=sliderCr->Value()/100.0f;
	theDescriptor.ci=sliderCi->Value()/100.0f;
	theDescriptor.cj=sliderCj->Value()/100.0f;
	theDescriptor.ck=sliderCk->Value()/100.0f;
	theDescriptor.wk=sliderWk->Value()/100.0f;
}